#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Cheat Sheet on files
# 
# Cheat sheet on files.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## change the encoding of a file

# In[2]:


with open("essai.txt", "w", encoding="latin-1") as f:
    f.write("ée\nàà")


# In[3]:


from ensae_projects.datainc import change_encoding
change_encoding("essai.txt", "essai.utf8.txt", enc1="latin-1", enc2="utf-8")


# In[4]:


with open("essai.utf8.txt", "r", encoding="utf8") as f:
    s = f.read()
print(s)


# ## select a subset of columns from a tsv files

# In[5]:


import pyensae.datasource
get_ipython().run_line_magic('load_ext', 'pyensae')
files = pyensae.datasource.download_data("OnlineNewsPopularity.zip", 
                              website="http://archive.ics.uci.edu/ml/machine-learning-databases/00332/")
files[1]


# In[6]:


get_ipython().run_line_magic('head', 'OnlineNewsPopularity/OnlineNewsPopularity.csv -n 2')


# In[7]:


from ensae_projects.datainc import enumerate_text_lines
def clean_column_name(s):
    return s.strip()
bigfile = enumerate_text_lines("OnlineNewsPopularity/OnlineNewsPopularity.csv", 
                               encoding="utf-8", header=True, quotes_as_str=False,
                               sep=",",
                               clean_column_name=clean_column_name, fLOG=print)


# In[8]:


res = list(map(lambda row: {"LDA_00": row["LDA_00"], "title_sentiment_polarity":row["title_sentiment_polarity"]}, 
            bigfile))
len(res)


# In[9]:


import pandas
df = pandas.DataFrame(res)
df.head()


# ## look at the head or tail of a file
# 
# We use magic commands [%head](http://www.xavierdupre.fr/app/pyensae/helpsphinx/all_NB.html?highlight=head#head) and [%tail](http://www.xavierdupre.fr/app/pyensae/helpsphinx/all_NB.html?highlight=head#tail).

# In[10]:


get_ipython().run_line_magic('load_ext', 'pyensae')


# In[11]:


get_ipython().run_line_magic('head', 'essai.txt -n 1 -s ignore')


# In[12]:


get_ipython().run_line_magic('tail', 'essai.txt -n 1 -s ignore')


# ## select lines of a flat file based on a regular expression

# In[13]:


get_ipython().run_line_magic('load_ext', 'pyensae')


# In[14]:


get_ipython().run_line_magic('grep', 'essai.utf8.txt .*é.*')


# More complex, we extract all lines containing a substring and we add the header to the file to make it look like a dataframe.
# We do that usually when we cannot load a big file into memory with [pandas](http://pandas.pydata.org/) due to the lack of memory. This code relies on magic command [grep](http://www.xavierdupre.fr/app/pyensae/helpsphinx/all_NB.html#grep) and function [enumerate_grep](http://www.xavierdupre.fr/app/pyensae/helpsphinx/pyensae/file_helper/content_helper.html?highlight=enumerate_grep#pyensae.file_helper.content_helper.enumerate_grep).

# In[15]:


import pandas
import pyensae
df = pandas.DataFrame([dict(name="Dupré", first_name="Xavier"),
                       dict(name="Dupré", first_name="Sloane")])
df.to_csv("data.txt", encoding="utf8", index=False)
get_ipython().run_line_magic('head', 'data.txt')


# In[16]:


raw = get_ipython().run_line_magic('grep', 'data.txt Xavier --raw')


# In[17]:


raw


# In[18]:


header = get_ipython().run_line_magic('head', 'data.txt -n 1 --raw')
header


# In[19]:


with open("data_xavier.txt", "w", encoding="utf8") as f:
    f.write(header)
    f.write(raw)
    
get_ipython().run_line_magic('head', 'data_xavier.txt')


# In[20]:


pandas.read_csv("data_xavier.txt")


# In[21]: